(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '../patch/createJsonPointer'], factory);
    }
})(function (require, exports) {
    "use strict";
    var createJsonPointer_1 = require('../patch/createJsonPointer');
    function createSort(comparatorOrProperty, descending, serializer) {
        var isFunction = typeof comparatorOrProperty === 'function';
        var comparator;
        if (isFunction) {
            comparator = comparatorOrProperty;
        }
        else {
            var pointer_1;
            if (typeof comparatorOrProperty === 'string') {
                pointer_1 = createJsonPointer_1.default(comparatorOrProperty);
            }
            else {
                pointer_1 = comparatorOrProperty;
            }
            comparator = function (a, b) {
                return sortValue(createJsonPointer_1.navigate(pointer_1, a), createJsonPointer_1.navigate(pointer_1, b));
            };
        }
        if (descending) {
            comparator = flip(comparator);
        }
        return {
            apply: function (data) {
                return data.sort(comparator);
            },
            comparatorOrProperty: comparatorOrProperty,
            descending: descending,
            queryType: 1 /* Sort */,
            toString: function (sortSerializer) {
                if (isFunction) {
                    throw Error('Cannot parse this sort type to an RQL query string');
                }
                return (sortSerializer || serializer || serialize)(this);
            },
            incremental: true
        };
    }
    function flip(comparator) {
        return function (a, b) {
            return -1 * comparator(a, b);
        };
    }
    function serialize(sort) {
        return "Sort(" + sort.comparatorOrProperty + ", " + (sort.descending ? '-' : '+') + ")";
    }
    // the `a == null` check returns `true` when a is `null` or `undefined`.
    function sortValue(a, b) {
        var comparison;
        if (a == null && b == null) {
            comparison = 0;
        }
        else if (a == null && b != null) {
            comparison = -1;
        }
        else if (b == null && a != null) {
            comparison = 1;
        }
        else if (a < b) {
            comparison = -1;
        }
        else if (a > b) {
            comparison = 1;
        }
        else {
            comparison = 0;
        }
        return comparison;
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createSort;
});
//# sourceMappingURL=createSort.js.map