(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-compose/compose', 'dojo-shim/Promise', 'dojo-shim/WeakMap', 'dojo-shim/Map', 'dojo-core/lang'], factory);
    }
})(function (require, exports) {
    "use strict";
    var compose_1 = require('dojo-compose/compose');
    var Promise_1 = require('dojo-shim/Promise');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var Map_1 = require('dojo-shim/Map');
    var lang_1 = require('dojo-core/lang');
    var instanceStateMap = new WeakMap_1.default();
    var createInMemoryStorage = compose_1.default({
        identify: function (items) {
            var state = instanceStateMap.get(this);
            var itemArray = Array.isArray(items) ? items : [items];
            if (state.idProperty) {
                var idProperty_1 = state.idProperty;
                return itemArray.map(function (item) {
                    return item[idProperty_1];
                });
            }
            else if (state.idFunction) {
                return itemArray.map(state.idFunction);
            }
            else {
                return itemArray.map(function (item) {
                    return item.id;
                });
            }
        },
        createId: function () {
            var state = instanceStateMap.get(this);
            return Promise_1.default.resolve(String(state.nextId++));
        },
        fetch: function (query) {
            var state = instanceStateMap.get(this);
            var data = state.data || [];
            return Promise_1.default.resolve((query ? query.apply(data) : data).slice());
        },
        get: function (ids) {
            var state = instanceStateMap.get(this);
            var data = state.data || [];
            var objects = [];
            return Promise_1.default.resolve(ids.reduce(function (prev, next) {
                return state.index.has(next) ? prev.concat(data[state.index.get(next)]) : prev;
            }, objects));
        },
        put: function (items, options) {
            var state = instanceStateMap.get(this);
            var ids = this.identify(items);
            var updatedItems = [];
            var oldIndices = [];
            var newIds = [];
            var newItems = [];
            ids.forEach(function (id, index) {
                var oldIndex = state.index.get(id);
                if (typeof oldIndex === 'undefined') {
                    newIds.push(id);
                    newItems.push(items[index]);
                }
                else {
                    updatedItems.push(items[index]);
                    oldIndices.push(oldIndex);
                }
            });
            if (oldIndices.length && options && options.rejectOverwrite) {
                return Promise_1.default.reject(new Error('Objects already exist in store'));
            }
            var data = state.data || (state.data = []);
            updatedItems.forEach(function (item, index) {
                data[oldIndices[index]] = item;
            });
            newItems.forEach(function (item, index) {
                state.index.set(newIds[index], data.push(item) - 1);
            });
            return Promise_1.default.resolve({
                successfulData: items,
                type: 1 /* Put */
            });
        },
        add: function (items, options) {
            options = options || {};
            if (typeof options.rejectOverwrite === 'undefined') {
                options.rejectOverwrite = true;
            }
            return this.put(items, options).then(function (result) {
                result.type = 0 /* Add */;
                return result;
            });
        },
        delete: function (ids) {
            var state = instanceStateMap.get(this);
            var data = state.data || [];
            var idsToRemove = ids.filter(function (id) {
                return state.index.has(id);
            });
            var indices = idsToRemove
                .map(function (id) {
                return state.index.get(id);
            })
                .sort();
            idsToRemove.forEach(function (id) {
                state.index.delete(id);
            });
            indices.forEach(function (index, indexArrayIndex) {
                return data.splice(index - indexArrayIndex, 1);
            });
            if (indices.length) {
                var firstInvalidIndex_1 = indices[0];
                var updateIndexForIds = this.identify(data.slice(firstInvalidIndex_1));
                updateIndexForIds.forEach(function (id, index) {
                    state.index.set(id, index + firstInvalidIndex_1);
                });
            }
            return Promise_1.default.resolve({
                successfulData: idsToRemove,
                type: 3 /* Delete */
            });
        },
        patch: function (updates) {
            var state = instanceStateMap.get(this);
            var data = state.data || (state.data = []);
            var filteredUpdates = updates.filter(function (update) {
                return state.index.has(update.id);
            });
            var oldIndices = filteredUpdates.map(function (update) {
                return state.index.get(update.id);
            });
            // If there is a source, the data has already been patched so we only need to sort it
            try {
                var updatedItems = filteredUpdates.map(function (update, index) {
                    var item = lang_1.duplicate(data[oldIndices[index]]);
                    var updatedItem = update.patch.apply(item);
                    data[oldIndices[index]] = updatedItem;
                    return updatedItem;
                });
                return Promise_1.default.resolve({
                    successfulData: updatedItems,
                    type: 2 /* Patch */
                });
            }
            catch (error) {
                return Promise_1.default.reject(error);
            }
        },
        isUpdate: function (item) {
            var state = instanceStateMap.get(this);
            var id = this.identify([item])[0];
            var isUpdate = state.index.has(id);
            return Promise_1.default.resolve({
                id: id,
                item: item,
                isUpdate: isUpdate
            });
        }
    }, function (instance, options) {
        options = options || {};
        instanceStateMap.set(instance, {
            data: [],
            nextId: 1,
            index: new Map_1.default(),
            idProperty: options.idProperty,
            idFunction: options.idFunction
        });
    });
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createInMemoryStorage;
});
//# sourceMappingURL=createInMemoryStorage.js.map