(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-shim/Promise', 'dojo-shim/WeakMap', 'dojo-shim/Map', 'dojo-core/lang', 'dojo-compose/compose', 'rxjs/Rx', '../patch/createPatch', './createStoreObservable', '../storage/createInMemoryStorage'], factory);
    }
})(function (require, exports) {
    "use strict";
    var Promise_1 = require('dojo-shim/Promise');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var Map_1 = require('dojo-shim/Map');
    var lang_1 = require('dojo-core/lang');
    var compose_1 = require('dojo-compose/compose');
    var Rx_1 = require('rxjs/Rx');
    var createPatch_1 = require('../patch/createPatch');
    var createStoreObservable_1 = require('./createStoreObservable');
    var createInMemoryStorage_1 = require('../storage/createInMemoryStorage');
    var instanceStateMap = new WeakMap_1.default();
    function isPatchArray(patches) {
        return isPatch(patches[0]);
    }
    function isPatch(patchObj) {
        var patch = patchObj && patchObj.patch;
        var id = patchObj && patchObj.id;
        return typeof id === 'string' && patch && Array.isArray(patch.operations) && typeof patch.apply === 'function' &&
            typeof patch.toString === 'function';
    }
    var createStore = compose_1.default({
        get: function (ids) {
            var state = instanceStateMap.get(this);
            return state.initialAddPromise.then(function () {
                return state.storage.get(Array.isArray(ids) ? ids : [ids]);
            });
        },
        add: function (items, options) {
            var self = this;
            var state = instanceStateMap.get(self);
            var storeResultsPromise = state.initialAddPromise.then(function () {
                return state.storage.add(Array.isArray(items) ? items : [items], options);
            });
            // TODO refactoring - repetitive logic
            return createStoreObservable_1.default(new Rx_1.Observable(function subscribe(observer) {
                storeResultsPromise
                    .then(function (results) {
                    observer.next(results);
                    observer.complete();
                }, function (error) {
                    observer.error(error);
                });
            }), function (results) {
                return results.successfulData;
            });
        },
        put: function (items, options) {
            var self = this;
            var state = instanceStateMap.get(self);
            var storeResultsPromise = state.initialAddPromise.then(function () {
                return state.storage.put(Array.isArray(items) ? items : [items], options);
            });
            return createStoreObservable_1.default(new Rx_1.Observable(function subscribe(observer) {
                storeResultsPromise
                    .then(function (results) {
                    observer.next(results);
                    observer.complete();
                }, function (error) {
                    observer.error(error);
                });
            }), function (results) {
                return results.successfulData;
            });
        },
        patch: function (updates, options) {
            var self = this;
            var state = instanceStateMap.get(self);
            var patchEntries = [];
            if (Array.isArray(updates)) {
                if (isPatchArray(updates)) {
                    patchEntries = updates;
                }
                else {
                    patchEntries = self.identify(updates).map(function (id, index) {
                        return { id: id, patch: createPatch_1.diff(updates[index]) };
                    });
                }
            }
            else if (updates instanceof Map_1.default) {
                updates.forEach(function (value, key) {
                    patchEntries.push({
                        id: key,
                        patch: value
                    });
                });
            }
            else if (isPatch(updates)) {
                patchEntries = [updates];
            }
            else {
                var dupe = lang_1.duplicate(updates);
                var idInOptions = (options && options.id);
                var id = idInOptions || dupe.id;
                if (!idInOptions) {
                    delete dupe.id;
                }
                patchEntries = [{ id: id, patch: createPatch_1.diff(dupe) }];
            }
            var storeResultsPromise = state.initialAddPromise.then(function () {
                return state.storage.patch(patchEntries);
            });
            return createStoreObservable_1.default(new Rx_1.Observable(function subscribe(observer) {
                storeResultsPromise
                    .then(function (results) {
                    observer.next(results);
                    observer.complete();
                }, function (error) {
                    observer.error(error);
                });
            }), function (results) {
                return results.successfulData;
            });
        },
        delete: function (ids) {
            var self = this;
            var state = instanceStateMap.get(self);
            var storeResultsPromise = state.initialAddPromise.then(function () {
                return state.storage.delete(Array.isArray(ids) ? ids : [ids]);
            });
            return createStoreObservable_1.default(new Rx_1.Observable(function subscribe(observer) {
                storeResultsPromise
                    .then(function (results) {
                    observer.next(results);
                    observer.complete();
                }, function (error) {
                    observer.error(error);
                });
            }), function (results) {
                return results.successfulData;
            });
        },
        fetch: function (query) {
            var state = instanceStateMap.get(this);
            return state.initialAddPromise.then(function () {
                return state.storage.fetch(query);
            });
        },
        identify: function (items) {
            return instanceStateMap.get(this).storage.identify(Array.isArray(items) ? items : [items]);
        },
        createId: function () {
            return instanceStateMap.get(this).storage.createId();
        }
    }, function (instance, options) {
        options = options || {};
        var data = options.data;
        options.data = undefined;
        var instanceState = {
            storage: options.storage || createInMemoryStorage_1.default(options),
            initialAddPromise: Promise_1.default.resolve()
        };
        instanceStateMap.set(instance, instanceState);
        if (data) {
            instanceState.initialAddPromise = instance.add(data).then(null, function () { });
        }
    });
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createStore;
});
//# sourceMappingURL=createStore.js.map