(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", './createStore', 'dojo-shim/WeakMap', 'dojo-core/lang'], factory);
    }
})(function (require, exports) {
    "use strict";
    var createStore_1 = require('./createStore');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var lang_1 = require('dojo-core/lang');
    var instanceStateMap = new WeakMap_1.default();
    var createSubcollectionStore = createStore_1.default
        .extend({
        get source() {
            var state = instanceStateMap.get(this);
            return state && state.source;
        },
        get factory() {
            var state = instanceStateMap.get(this);
            return state && state.factory;
        },
        createSubcollection: function (options) {
            // Need to reassign the factory or compose throws an error for instantiating
            // with new
            var factory = this.factory;
            var defaultOptions = this.getOptions();
            return factory(lang_1.mixin(defaultOptions, options || {}));
        },
        getOptions: function () {
            return {
                source: this.source || this,
                // Provide something to prevent the Subcollection from instantiating its own storage. The type doesn't
                // matter because it'll never be used.
                storage: true
            };
        }
    }).mixin({
        initialize: function (instance, options) {
            options = options || {};
            instanceStateMap.set(instance, {
                source: options.source,
                factory: instance.constructor
            });
        },
        aspectAdvice: {
            around: {
                get: function (get) {
                    return function (ids) {
                        if (this.source) {
                            return this.source.get(ids);
                        }
                        else {
                            return get.call(this, ids);
                        }
                    };
                },
                add: function (add) {
                    return function (items, options) {
                        if (this.source) {
                            return this.source.add(items, options);
                        }
                        else {
                            return add.call(this, items, options);
                        }
                    };
                },
                delete: function (_delete) {
                    return function (ids) {
                        if (this.source) {
                            return this.source.delete(ids);
                        }
                        else {
                            return _delete.call(this, ids);
                        }
                    };
                },
                put: function (put) {
                    return function (items, options) {
                        if (this.source) {
                            return this.source.put(items, options);
                        }
                        else {
                            return put.call(this, items, options);
                        }
                    };
                },
                patch: function (patch) {
                    return function (updates, options) {
                        if (this.source) {
                            return this.source.patch(updates, options);
                        }
                        else {
                            return patch.call(this, updates, options);
                        }
                    };
                },
                fetch: function (fetch) {
                    return function (query) {
                        if (this.source) {
                            return this.source.fetch(query);
                        }
                        else {
                            return fetch.call(this, query);
                        }
                    };
                },
                identify: function (identify) {
                    return function (items) {
                        if (this.source) {
                            return this.source.identify(items);
                        }
                        else {
                            return identify.call(this, items);
                        }
                    };
                },
                createId: function (createId) {
                    return function (items) {
                        if (this.source) {
                            return this.source.createId();
                        }
                        else {
                            return createId.call(this);
                        }
                    };
                }
            }
        }
    });
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createSubcollectionStore;
});
//# sourceMappingURL=createSubcollectionStore.js.map