import { CrudOptions, Store, StoreOptions, UpdateResults } from '../createStore';
import { Observable, Observer } from 'rxjs/Rx';
import Map from 'dojo-shim/Map';
import Set from 'dojo-shim/Set';
import { ComposeMixinDescriptor } from 'dojo-compose/compose';
export interface StoreDelta<T> {
    updates: T[];
    deletes: string[];
    adds: T[];
    beforeAll?: T[];
    afterAll?: T[];
}
export interface ItemUpdate<T> {
    item?: T;
    id: string;
}
export interface ObservableStoreMixin<T> {
    observe(): Observable<StoreDelta<T>>;
    observe(id: string): Observable<T>;
    observe(ids: string[]): Observable<ItemUpdate<T>>;
}
export interface ObservableStoreMixinOptions<T> {
    fetchAroundUpdates?: boolean;
    scheduleUpdates?: (delta: StoreDelta<T>, sendUpdates: () => void) => void;
}
export declare type ObserverSetEntry<T> = {
    observes: Set<string>;
    observer: Observer<ItemUpdate<T>>;
};
export interface ObservableStoreState<T> {
    fetchAroundUpdates: boolean;
    itemObservers: Map<string, (Observer<T> | ObserverSetEntry<T>)[]>;
    toRemoveIndices: number[];
    observers: Observer<StoreDelta<T>>[];
    storeObservable: Observable<StoreDelta<T>>;
    updates: T[];
    deletes: string[];
    adds: T[];
    beforeAll?: T[];
    scheduleUpdates: (delta: StoreDelta<T>, sendUpdates: () => void) => void;
}
export interface ObservableStore<T, O extends CrudOptions, U extends UpdateResults<T>> extends ObservableStoreMixin<T>, Store<T, O, U> {
}
export declare type ObservableStoreOptions<T, O extends CrudOptions> = ObservableStoreMixinOptions<T> & StoreOptions<T, O>;
declare function createObservableStoreMixin<T, O extends CrudOptions, U extends UpdateResults<T>>(): ComposeMixinDescriptor<Store<T, O, U>, CrudOptions, ObservableStoreMixin<T>, ObservableStoreMixinOptions<T>>;
export default createObservableStoreMixin;
