(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'rxjs/Rx', 'dojo-shim/WeakMap', 'dojo-shim/Map', 'dojo-shim/Set', 'dojo-shim/Promise', 'dojo-compose/aspect'], factory);
    }
})(function (require, exports) {
    "use strict";
    var Rx_1 = require('rxjs/Rx');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var Map_1 = require('dojo-shim/Map');
    var Set_1 = require('dojo-shim/Set');
    var Promise_1 = require('dojo-shim/Promise');
    var aspect_1 = require('dojo-compose/aspect');
    var instanceStateMap = new WeakMap_1.default();
    function sendUpdates(after) {
        var state = instanceStateMap.get(this);
        function send(after) {
            var storeDelta = {
                updates: state.updates.splice(0),
                deletes: state.deletes.splice(0),
                adds: state.adds.splice(0),
                beforeAll: state.beforeAll,
                afterAll: after
            };
            state.beforeAll = after;
            state.observers.forEach(function (observer) {
                observer.next(storeDelta);
            });
            state.toRemoveIndices.splice(0).sort().reverse().forEach(function (removeIndex) {
                state.observers.splice(removeIndex, 1);
            });
        }
        after.then(function (data) {
            send(data);
        });
    }
    function isObserverEntry(observer) {
        return observer.observes instanceof Set_1.default;
    }
    function isObserver(observer) {
        return !isObserverEntry(observer);
    }
    function notifyItemObservers(items, ids, state, store) {
        function notify(id, after) {
            if (state.itemObservers.has(id)) {
                state.itemObservers.get(id).map(function (observerOrEntry) {
                    if (isObserverEntry(observerOrEntry)) {
                        return observerOrEntry.observer;
                    }
                    else {
                        return null;
                    }
                }).filter(function (observerEntry) {
                    return observerEntry;
                }).forEach(function (observer) {
                    observer.next({
                        item: after,
                        id: id
                    });
                });
                if (after) {
                    state.itemObservers.get(id).map(function (observerOrEntry) {
                        if (isObserver(observerOrEntry)) {
                            return observerOrEntry;
                        }
                        else {
                            return null;
                        }
                    }).filter(function (observer) {
                        return observer;
                    }).forEach(function (observer) {
                        observer.next(after);
                    });
                }
            }
        }
        if (items) {
            items.forEach(function (after, index) {
                var id = ids[index] || store.identify(after)[0];
                notify(id, after);
            });
        }
        else {
            ids.forEach(function (id) {
                notify(id, undefined);
            });
        }
    }
    function isSubcollectionStore(store) {
        return Boolean(store.createSubcollection);
    }
    function createObservableStoreMixin() {
        return {
            mixin: {
                observe: function (idOrIds) {
                    if (isSubcollectionStore(this)) {
                        var subcollectionStore = this;
                        if (subcollectionStore.source) {
                            return subcollectionStore.source.observe(idOrIds);
                        }
                    }
                    if (idOrIds) {
                        var self_1 = this;
                        var state_1 = instanceStateMap.get(self_1);
                        if (Array.isArray(idOrIds)) {
                            var ids_1 = idOrIds;
                            var idSet_1 = new Set_1.default(ids_1);
                            var observable = new Rx_1.Observable(function subscribe(observer) {
                                var observerEntry = {
                                    observes: idSet_1,
                                    observer: observer
                                };
                                ids_1.forEach(function (id) {
                                    if (state_1.itemObservers.has(id)) {
                                        state_1.itemObservers.get(id).push(observerEntry);
                                    }
                                    else {
                                        state_1.itemObservers.set(id, [observerEntry]);
                                    }
                                });
                                var isStarted = false;
                                observer.next = aspect_1.after(observer.next, function (result) {
                                    isStarted = true;
                                    return result;
                                });
                                observer.error = aspect_1.after(observer.error, function (result) {
                                    isStarted = true;
                                    return result;
                                });
                                observer.complete = aspect_1.after(observer.complete, function (result) {
                                    isStarted = true;
                                    return result;
                                });
                                self_1.get(ids_1).then(function (items) {
                                    if (!isStarted) {
                                        var retrievedIdSet_1 = new Set_1.default(self_1.identify(items));
                                        var missingItemIds = ids_1.filter(function (id) { return !retrievedIdSet_1.has(id); });
                                        if (retrievedIdSet_1.size !== idSet_1.size || missingItemIds.length) {
                                            observer.error(new Error("ID(s) \"" + missingItemIds + "\" not found in store"));
                                        }
                                        else {
                                            items.forEach(function (item, index) { return observer.next({
                                                item: item,
                                                id: ids_1[index]
                                            }); });
                                        }
                                    }
                                });
                            });
                            return observable;
                        }
                        else {
                            var id_1 = idOrIds;
                            return new Rx_1.Observable(function subscribe(observer) {
                                self_1.get(id_1).then(function (items) {
                                    var item = items[0];
                                    if (!item) {
                                        observer.error(new Error("ID \"" + id_1 + "\" not found in store"));
                                    }
                                    else {
                                        if (state_1.itemObservers.has(id_1)) {
                                            state_1.itemObservers.get(id_1).push(observer);
                                        }
                                        else {
                                            state_1.itemObservers.set(id_1, [observer]);
                                        }
                                        observer.next(item);
                                    }
                                });
                            });
                        }
                    }
                    else {
                        return instanceStateMap.get(this).storeObservable;
                    }
                }
            },
            aspectAdvice: {
                after: {
                    put: function (result) {
                        var self = this;
                        var state = instanceStateMap.get(self);
                        var fetchedDataPromise;
                        if (state.fetchAroundUpdates) {
                            fetchedDataPromise = self.fetch();
                        }
                        else {
                            fetchedDataPromise = Promise_1.default.resolve(undefined);
                        }
                        result.then(function (updatedItems) {
                            notifyItemObservers(updatedItems, [], state, self);
                            state.updates = state.updates.concat(updatedItems);
                            state.scheduleUpdates(state, sendUpdates.bind(self, fetchedDataPromise));
                        });
                        return result;
                    },
                    patch: function (result) {
                        var self = this;
                        var state = instanceStateMap.get(self);
                        var fetchedDataPromise;
                        if (state.fetchAroundUpdates) {
                            fetchedDataPromise = self.fetch();
                        }
                        else {
                            fetchedDataPromise = Promise_1.default.resolve(undefined);
                        }
                        result.then(function (updatedItems) {
                            notifyItemObservers(updatedItems, [], state, self);
                            state.updates = state.updates.concat(updatedItems);
                            state.scheduleUpdates(state, sendUpdates.bind(self, fetchedDataPromise));
                        });
                        return result;
                    },
                    add: function (result) {
                        var self = this;
                        var state = instanceStateMap.get(self);
                        if (state) {
                            var fetchedDataPromise_1;
                            if (state.fetchAroundUpdates) {
                                fetchedDataPromise_1 = self.fetch();
                            }
                            else {
                                fetchedDataPromise_1 = Promise_1.default.resolve(undefined);
                            }
                            result.then(function (addedItems) {
                                notifyItemObservers(addedItems, [], state, self);
                                state.adds = state.adds.concat(addedItems);
                                state.scheduleUpdates(state, sendUpdates.bind(self, fetchedDataPromise_1));
                            });
                        }
                        return result;
                    },
                    delete: function (result, ids) {
                        var self = this;
                        var state = instanceStateMap.get(self);
                        var fetchedDataPromise;
                        if (state.fetchAroundUpdates) {
                            fetchedDataPromise = self.fetch();
                        }
                        else {
                            fetchedDataPromise = Promise_1.default.resolve(undefined);
                        }
                        result.then(function (deleted) {
                            notifyItemObservers(null, deleted, state, self);
                            deleted.forEach(function (id) {
                                if (state.itemObservers.has(id)) {
                                    state.itemObservers.get(id).forEach(function (observerOrEntry) {
                                        if (isObserverEntry(observerOrEntry)) {
                                            observerOrEntry.observes.delete(id);
                                            if (!observerOrEntry.observes.size) {
                                                observerOrEntry.observer.complete();
                                            }
                                        }
                                        else if (isObserver(observerOrEntry)) {
                                            observerOrEntry.complete();
                                        }
                                    });
                                    state.itemObservers.delete(id);
                                }
                            });
                            state.deletes = state.deletes.concat(deleted);
                            state.scheduleUpdates(state, sendUpdates.bind(self, fetchedDataPromise));
                        });
                        return result;
                    }
                }
            },
            initialize: function (instance, options) {
                options = options || {};
                var itemObservers = new Map_1.default();
                var storeObservable = new Rx_1.Observable(function (observer) {
                    var state = instanceStateMap.get(this);
                    state.observers.push(observer);
                    return function () {
                        return state.toRemoveIndices.push(state.observers.indexOf(observer));
                    };
                }.bind(instance));
                var state = {
                    fetchAroundUpdates: Boolean(options.fetchAroundUpdates),
                    scheduleUpdates: options.scheduleUpdates || function (storeDelta, sendUpdates) {
                        sendUpdates();
                    },
                    itemObservers: itemObservers,
                    toRemoveIndices: [],
                    observers: [],
                    storeObservable: storeObservable,
                    updates: [],
                    deletes: [],
                    adds: []
                };
                if (options.fetchAroundUpdates) {
                    instance.fetch().then(function (data) {
                        state.beforeAll = data;
                    });
                }
                if (isSubcollectionStore(instance)) {
                    var subcollectionStore = instance;
                    subcollectionStore.getOptions = aspect_1.after(subcollectionStore.getOptions, function (options) {
                        options = options || {};
                        options.fetchAroundUpdates = state.fetchAroundUpdates;
                        return options;
                    });
                }
                instanceStateMap.set(instance, state);
            }
        };
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createObservableStoreMixin;
});
//# sourceMappingURL=createObservableStoreMixin.js.map