(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'dojo-shim/WeakMap', 'dojo-shim/Promise', 'rxjs/Rx', '../createStoreObservable'], factory);
    }
})(function (require, exports) {
    "use strict";
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var Promise_1 = require('dojo-shim/Promise');
    var Rx_1 = require('rxjs/Rx');
    var createStoreObservable_1 = require('../createStoreObservable');
    var instanceStateMap = new WeakMap_1.default();
    function processNext(state) {
        var operation = state.operationQueue.shift();
        if (!operation) {
            state.inProgress = false;
            return;
        }
        var promiseOrObservable = operation();
        if (promiseOrObservable.toPromise) {
            promiseOrObservable.subscribe(null, function () {
                processNext(state);
            }, function () {
                processNext(state);
            });
        }
        else if (promiseOrObservable.then) {
            promiseOrObservable.then(function () {
                processNext(state);
            }, function () {
                processNext(state);
            });
        }
    }
    function queueStoreOperation(operation, returnsPromise) {
        return function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i - 0] = arguments[_i];
            }
            var store = this;
            var state = instanceStateMap.get(store);
            if (!state || this.source) {
                return operation.apply(store, args);
            }
            if (!state.inProgress) {
                state.inProgress = true;
                setTimeout(function () {
                    processNext(state);
                });
            }
            if (returnsPromise) {
                return new Promise_1.default(function (resolve, reject) {
                    state.operationQueue.push(function () {
                        return operation.apply(store, args).then(function (results) {
                            resolve(results);
                        }, function (error) {
                            reject(error);
                        });
                    });
                });
            }
            else {
                var pushed_1 = false;
                var observers_1 = [];
                var operationObservable_1;
                var observable = createStoreObservable_1.default(new Rx_1.Observable(function subscribe(observer) {
                    if (!operationObservable_1) {
                        observers_1.push(observer);
                    }
                    if (!pushed_1) {
                        state.operationQueue.push(function () {
                            operationObservable_1 = operation.apply(store, args);
                            observers_1.forEach(function (observer) {
                                operationObservable_1.subscribe(observer);
                            });
                            return operationObservable_1;
                        });
                        pushed_1 = true;
                    }
                    return function () { return observers_1.splice(observers_1.indexOf(observer), 1); };
                }), function (updateResults) {
                    return updateResults.successfulData;
                });
                observable.subscribe();
                return observable;
            }
        };
    }
    function createOrderedOperationMixin() {
        return {
            aspectAdvice: {
                around: {
                    put: function (put) {
                        return queueStoreOperation(put);
                    },
                    add: function (add) {
                        return queueStoreOperation(add);
                    },
                    patch: function (patch) {
                        return queueStoreOperation(patch);
                    },
                    delete: function (_delete) {
                        return queueStoreOperation(_delete);
                    },
                    fetch: function (fetch) {
                        return queueStoreOperation(fetch, true);
                    }
                }
            },
            initialize: function (instance) {
                instanceStateMap.set(instance, {
                    operationQueue: []
                });
            }
        };
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createOrderedOperationMixin;
});
//# sourceMappingURL=createOrderedOperationMixin.js.map