import { Query } from '../../query/interfaces';
import { Filter } from '../../query/createFilter';
import { StoreRange } from '../../query/createStoreRange';
import { CrudOptions, Store, UpdateResults } from '../createStore';
import { Sort } from '../../query/createSort';
import { SubcollectionStore, SubcollectionOptions } from '../createSubcollectionStore';
import { ComposeMixinDescriptor } from 'dojo-compose/compose';
export interface QueryMixin<T, O extends CrudOptions, U extends UpdateResults<T>, C extends Store<T, O, U>> {
    query(query: Query<T, T>): C & this;
    filter(filter: Filter<T>): C & this;
    filter(test: (item: T) => boolean): C & this;
    range(range: StoreRange<T>): C & this;
    range(start: number, count: number): C & this;
    sort(sort: Sort<T> | ((a: T, b: T) => number) | string, descending?: boolean): C & this;
}
export declare type QueryStore<T, O extends CrudOptions, U extends UpdateResults<T>, C extends Store<T, O, U>> = QueryMixin<T, O, U, C> & C;
export interface QueryOptions<T> {
    sourceQuery?: Query<T, T>;
}
declare function createQueryMixin<T, O extends CrudOptions, U extends UpdateResults<T>, C extends Store<T, O, U>>(): ComposeMixinDescriptor<SubcollectionStore<T, O, U, any>, SubcollectionOptions<T, O, U>, QueryMixin<T, O, U, C>, QueryOptions<T>>;
export default createQueryMixin;
