(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '../../query/createCompoundQuery', '../../query/createFilter', '../../query/createStoreRange', '../../query/createSort', 'dojo-shim/WeakMap'], factory);
    }
})(function (require, exports) {
    "use strict";
    var createCompoundQuery_1 = require('../../query/createCompoundQuery');
    var createFilter_1 = require('../../query/createFilter');
    var createStoreRange_1 = require('../../query/createStoreRange');
    var createSort_1 = require('../../query/createSort');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var instanceStateMap = new WeakMap_1.default();
    function isFilter(filterOrTest) {
        return typeof filterOrTest !== 'function' && filterOrTest.queryType === 0 /* Filter */;
    }
    function isSort(sortOrComparator) {
        var paramType = typeof sortOrComparator;
        return paramType !== 'function' && paramType !== 'string' && typeof sortOrComparator.apply === 'function';
    }
    function createQueryMixin() {
        var queryMixin = {
            query: function (query) {
                var state = instanceStateMap.get(this);
                if (state.sourceQuery) {
                    var compoundQuery = state.sourceQuery.queryType === 3 /* Compound */ ?
                        state.sourceQuery : createCompoundQuery_1.default({ query: state.sourceQuery });
                    query = compoundQuery.withQuery(query);
                }
                return this.createSubcollection({
                    sourceQuery: query
                });
            },
            filter: function (filterOrTest) {
                var filter;
                if (isFilter(filterOrTest)) {
                    filter = filterOrTest;
                }
                else {
                    filter = createFilter_1.default().custom(filterOrTest);
                }
                return this.query(filter);
            },
            range: function (rangeOrStart, count) {
                var range;
                if (typeof count !== 'undefined') {
                    range = createStoreRange_1.default(rangeOrStart, count);
                }
                else {
                    range = rangeOrStart;
                }
                return this.query(range);
            },
            sort: function (sortOrComparator, descending) {
                var sort;
                if (isSort(sortOrComparator)) {
                    sort = sortOrComparator;
                }
                else {
                    sort = createSort_1.default(sortOrComparator, descending);
                }
                return this.query(sort);
            }
        };
        return {
            mixin: queryMixin,
            aspectAdvice: {
                before: {
                    fetch: function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i - 0] = arguments[_i];
                        }
                        var state = instanceStateMap.get(this);
                        var query = args[0];
                        if (state && state.sourceQuery) {
                            var sourceQuery = state.sourceQuery;
                            if (query) {
                                var compoundQuery = sourceQuery.queryType === 3 /* Compound */ ?
                                    sourceQuery : createCompoundQuery_1.default({ query: sourceQuery });
                                query = compoundQuery.withQuery(query);
                            }
                            else {
                                query = sourceQuery;
                            }
                            args[0] = query;
                        }
                        return args;
                    },
                    createSubcollection: function (options) {
                        options = options || {};
                        var state = instanceStateMap.get(this);
                        if (!options.sourceQuery && state.sourceQuery) {
                            options.sourceQuery = state.sourceQuery;
                        }
                        return [options];
                    }
                }
            },
            initialize: function (instance, options) {
                options = options || {};
                instanceStateMap.set(instance, {
                    sourceQuery: options.sourceQuery
                });
            }
        };
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createQueryMixin;
});
//# sourceMappingURL=createQueryMixin.js.map