import { SubcollectionStore, SubcollectionOptions } from '../createSubcollectionStore';
import { CrudOptions, UpdateResults } from '../createStore';
import { ObservableStore, StoreDelta } from './createObservableStoreMixin';
import { Observable } from 'rxjs/Rx';
import { ComposeMixinDescriptor } from 'dojo-compose/compose';
import { Query } from '../../query/interfaces';
export interface TrackedStoreDelta<T> extends StoreDelta<T> {
    removedFromTracked: {
        item: T;
        id: string;
        previousIndex: number;
    }[];
    addedToTracked: {
        item: T;
        id: string;
        index: number;
    }[];
    movedInTracked: {
        item: T;
        id: string;
        previousIndex: number;
        index: number;
    }[];
}
export interface TrackedObservableStoreMixin<T> {
    observe(): Observable<TrackedStoreDelta<T>>;
}
export interface TrackableMixin<T, O extends CrudOptions, U extends UpdateResults<T>, C extends ObservableStore<T, O, U>> {
    track(): TrackedObservableStoreMixin<T> & C & this;
    release(): ObservableStore<T, O, U> & C & this;
}
export declare type TrackableStore<T, O extends CrudOptions, U extends UpdateResults<T>, C extends ObservableStore<T, O, U>> = TrackableMixin<T, O, U, C> & C;
export interface TrackableOptions<T> {
    isTracking?: boolean;
    sourceQuery?: Query<T, T>;
    fetchAroundUpdates?: boolean;
    wasFetchingAroundUpdates?: boolean;
}
declare function createTrackableMixin<T, O extends CrudOptions, U extends UpdateResults<T>, C extends ObservableStore<T, O, U>>(): ComposeMixinDescriptor<ObservableStore<T, O, U> & SubcollectionStore<T, O, U, any>, SubcollectionOptions<T, O, U>, TrackableMixin<T, O, U, C>, TrackableOptions<T>>;
export default createTrackableMixin;
