(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", 'rxjs/Rx', 'dojo-compose/aspect', 'dojo-shim/Map', 'dojo-shim/WeakMap'], factory);
    }
})(function (require, exports) {
    "use strict";
    var Rx_1 = require('rxjs/Rx');
    var aspect_1 = require('dojo-compose/aspect');
    var Map_1 = require('dojo-shim/Map');
    var WeakMap_1 = require('dojo-shim/WeakMap');
    var instanceStateMap = new WeakMap_1.default();
    function buildTrackedUpdate(state, store) {
        return function (update) {
            var removedFromTracked = [];
            var addedToTracked = [];
            var movedInTracked = [];
            // Handle deletes
            var deletedIndices = [];
            var trackedDeletes = update.deletes.filter(function (id) {
                if (state.idToIndex.has(id)) {
                    var index = state.idToIndex.get(id);
                    deletedIndices.push(index);
                    removedFromTracked.push({
                        item: state.localData[index],
                        id: id,
                        previousIndex: index
                    });
                    state.idToIndex.delete(id);
                    return true;
                }
            });
            var newData;
            if (update.afterAll) {
                newData = update.afterAll;
            }
            else {
                newData = state.localData.slice().concat(update.adds);
                store.identify(update.updates).forEach(function (id, index) {
                    if (state.idToIndex.has(id)) {
                        newData[state.idToIndex.get(id)] = update.updates[index];
                    }
                    else {
                        newData.push(update.updates[index]);
                    }
                });
                deletedIndices.sort().reverse().forEach(function (removeIndex) {
                    newData.splice(removeIndex, 1);
                });
            }
            if (state.sourceQuery) {
                newData = state.sourceQuery.apply(newData);
            }
            var trackedUpdates = [];
            var trackedAdds = [];
            var newIndex = new Map_1.default();
            var newIds = store.identify(newData);
            newIds.forEach(function (id, index) {
                newIndex.set(id, index);
            });
            var updateIds = store.identify(update.updates);
            updateIds.forEach(function (id, updateIndex) {
                if (!newIndex.has(id) && state.idToIndex.has(id)) {
                    trackedUpdates.push(update.updates[updateIndex]);
                    var index = state.idToIndex.get(id);
                    removedFromTracked.push({
                        item: update.updates[updateIndex],
                        previousIndex: index,
                        id: id
                    });
                }
                else if (newIndex.has(id) && state.idToIndex.has(id)) {
                    trackedUpdates.push(update.updates[updateIndex]);
                    var previouxIndex = state.idToIndex.get(id);
                    var index = newIndex.get(id);
                    movedInTracked.push({
                        item: newData[index],
                        id: id,
                        previousIndex: previouxIndex,
                        index: index
                    });
                }
            });
            var updateAndAddIds = updateIds.concat(store.identify(update.adds));
            updateAndAddIds.forEach(function (id, itemIndex) {
                if (newIndex.has(id) && !state.idToIndex.has(id)) {
                    if (itemIndex < update.updates.length) {
                        trackedUpdates.push(update.updates[itemIndex]);
                    }
                    else {
                        trackedAdds.push(update.adds[itemIndex - update.updates.length]);
                    }
                    trackedAdds.push();
                    var index = newIndex.get(id);
                    addedToTracked.push({
                        item: newData[index],
                        id: id,
                        index: index
                    });
                }
            });
            // Check for items pushed into or out of range
            newIds.forEach(function (id, index) {
                if (!state.idToIndex.has(id) && updateAndAddIds.indexOf(id) < 0) {
                    addedToTracked.push({
                        item: newData[index],
                        id: id,
                        index: newIndex.get(id)
                    });
                }
            });
            store.identify(state.localData).forEach(function (id, index) {
                if (!newIndex.has(id) && trackedDeletes.indexOf(id) < 0 && updateAndAddIds.indexOf(id) < 0) {
                    removedFromTracked.push({
                        item: state.localData[index],
                        id: id,
                        previousIndex: index
                    });
                }
            });
            var trackedUpdate = {
                beforeAll: state.localData,
                afterAll: newData,
                updates: trackedUpdates,
                adds: trackedAdds,
                deletes: trackedDeletes,
                removedFromTracked: removedFromTracked,
                movedInTracked: movedInTracked,
                addedToTracked: addedToTracked
            };
            // Don't send an update if nothing happened withing the scope of this tracked.
            if (trackedUpdates.length || trackedAdds.length || trackedDeletes.length || removedFromTracked.length ||
                movedInTracked.length || addedToTracked.length) {
                state.localData = newData;
                state.idToIndex = newIndex;
                state.observers.forEach(function (observer) {
                    observer.next(trackedUpdate);
                });
                state.toRemoveIndices.sort().reverse().forEach(function (index) {
                    state.observers.splice(index, 1);
                });
                state.toRemoveIndices = [];
            }
        };
    }
    function createTrackableMixin() {
        var TrackableMixin = {
            track: function () {
                var state = instanceStateMap.get(this);
                return this.createSubcollection({
                    isTracking: true,
                    fetchAroundUpdates: !state.sourceQuery || !state.sourceQuery.incremental,
                    wasFetchingAroundUpdates: state.fetchAroundUpdates || state.wasFetchingAroundUpdates
                });
            },
            release: function () {
                var state = instanceStateMap.get(this);
                return this.createSubcollection({
                    isTracking: false,
                    fetchAroundUpdates: state.wasFetchingAroundUpdates,
                    wasFetchingAroundUpdates: state.wasFetchingAroundUpdates
                });
            }
        };
        return {
            mixin: TrackableMixin,
            initialize: function (instance, options) {
                options = options || {};
                instanceStateMap.set(instance, {
                    isTracking: Boolean(options.isTracking),
                    sourceQuery: options.sourceQuery,
                    localData: [],
                    idToIndex: new Map_1.default(),
                    observers: [],
                    toRemoveIndices: [],
                    fetchAroundUpdates: options.fetchAroundUpdates,
                    wasFetchingAroundUpdates: options.wasFetchingAroundUpdates
                });
                var state = instanceStateMap.get(instance);
                if (options.isTracking && instance.source) {
                    instance.fetch().then(function (data) {
                        state.localData = data;
                        instance.identify(data).forEach(function (id, index) {
                            state.idToIndex.set(id, index);
                        });
                    });
                    state.observable = new Rx_1.Observable(function (observer) {
                        state.observers.push(observer);
                        return function () {
                            return state.toRemoveIndices.push(state.observers.indexOf(observer));
                        };
                    }.bind(instance));
                    instance.source.observe().subscribe(buildTrackedUpdate(state, instance));
                }
                var subcollectionStore = instance;
                subcollectionStore.getOptions = aspect_1.after(subcollectionStore.getOptions, function (options) {
                    var state = instanceStateMap.get(this);
                    options = options || {};
                    options.fetchAroundUpdates = state.fetchAroundUpdates || state.wasFetchingAroundUpdates;
                    return options;
                });
            },
            aspectAdvice: {
                around: {
                    observe: function (observe) {
                        return function (idOrIds) {
                            if (idOrIds || !instanceStateMap.get(this).isTracking) {
                                return observe.call(this, idOrIds);
                            }
                            else {
                                return instanceStateMap.get(this).observable;
                            }
                        };
                    }
                }
            }
        };
    }
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = createTrackableMixin;
});
//# sourceMappingURL=createTrackableMixin.js.map